// SPI-Schnittstelle   Dr. Weisser 22.2.2019  24.2.2019
//***************************************************************************
//  File........: spi.c
//  Target(s)...: ATmega328p, mega2560
//  Compiler....: AVR-GCC
//  Description.: SPI routines
//***************************************************************************

#include <stdint.h>     // Integer-Datentypen 8-64bit
#include <avr/io.h>     // Pins und Interrupt-Tabelle -> siehe auch iom328p.h
#include "main.h"       // CPU-Frequenz, sbi, cbi
#include "spi.h"        // Funktions-Prototypen



/*****************************************************************************
*   Function name : spi_init
*   Returns :       None
*   Parameters :    None
*   Purpose :       Initialize SPI
*                   SPI-mode 0 
*                   8 MHz bei 16 MHz-Taktfrequenz
*****************************************************************************/
void spi_init(void)
{
   // SPI Control-Register schreiben
   // kein Int, SPI deaktiv, bit-Reihenfolge, Master, Mode 0, SPI 8MHz bei 16 MHz-Takt
   //      bei SD-Card   SPI-Mode 0 od. 3 - lt. Chan: How to Use MMC_SDC.htm 
   //      bei MAX5717   SPI-Mode 1
   //      bei AD7714    SPI-Mode 1 (bei POL=0)
   //      bei AD7798    SPI-Mode 3
   //      bei 23LCV1024 SPI-Mode 0
   SPCR = (0<<SPIE)|(0<<SPE)|(0<<DORD)|(1<<MSTR)|(1<<CPOL)|(1<<CPHA)|(0<<SPR1)|(0<<SPR0);
        // kein SPI-Interrupt
        //           SPI deaktiv
        //                    MSB first
        //                              Master
        //                                        CPOL      CPHA
        //                                          0         0   SPI mode 0
        //                                          0         1   SPI mode 1
        //                                          1         0   SPI mode 2
        //                                          1         1   SPI mode 3
        //                                                            SPR1, SPR0 Takt-Teiler

   // SPI-Takt: SPI2X SPR1 SPR0
           //     1    0    0    fosc/2 = 8 MHz bei 16MHz-Takt
           //     0    0    0    fosc/4 = 4 MHz
           //     1    0    1    fosc/8 = 2 MHz
           //     0    0    1    fosc/16= 1 MHz
           //     1    1    0    fosc/32= 500kHz
           //     1    1    1    fosc/64= 250kHz
           //     0    1    1    fosc/128=125kHz

   // Takt der SPI verdoppeln
   SPSR = (1<<SPI2X);  // 8MHz bei 16 MHz-Takt
}



/*****************************************************************************
*   Function name : spi_speed8
*   Returns :       None
*   Parameters :    None
*   Purpose :       set SPI speed to 8 MHz
*****************************************************************************/
void spi_speed8(void)
{
   // SPI-Takt festlegen: SPI2X SPR1 SPR0
                     //     0    0    0    fosc/4 = 4 MHz bei 16MHz-Takt
                     //     1    0    0    fosc/2 = 8 MHz
                     //     1    0    1    fosc/8 = 2 MHz
                     //     0    0    1    fosc/16= 1 MHz
                     //     1    1    0    fosc/32= 500kHz
                     //     1    1    1    fosc/64= 250kHz
                     //     0    1    1    fosc/128=125kHz

   // SPI Control-Register: untere 2 bits SPR1, SPR0 fuer Takt-Teiler
   // SPCR = (0<<SPR1)|(0<<SPR0);

   // SPI Status-Register: unterstes bit SPI2X fuer Takt-Verdoppler
   // SPSR = (1<<SPI2X);

   // SPI 8MHz
   sbi(SPSR,SPI2X);
   cbi(SPCR,SPR1);
   cbi(SPCR,SPR0);
/*
   // SPI 4MHz
   cbi(SPSR,SPI2X);
   cbi(SPCR,SPR1);
   cbi(SPCR,SPR0);

   // SPI 2MHz
   sbi(SPSR,SPI2X);
   cbi(SPCR,SPR1);
   sbi(SPCR,SPR0);

   // SPI 1MHz
   cbi(SPSR,SPI2X);
   cbi(SPCR,SPR1);
   sbi(SPCR,SPR0);

   // SPI 500kHz
   sbi(SPSR,SPI2X);
   sbi(SPCR,SPR1);
   cbi(SPCR,SPR0);

   // SPI 250kHz
   sbi(SPSR,SPI2X);
   sbi(SPCR,SPR1);
   sbi(SPCR,SPR0);

   // SPI 125kHz
   cbi(SPSR,SPI2X);
   sbi(SPCR,SPR1);
   sbi(SPCR,SPR0);
*/

}



/*****************************************************************************
*   Function name : spi_speed4
*   Returns :       None
*   Parameters :    None
*   Purpose :       set SPI speed to 4 MHz
*****************************************************************************/
void spi_speed4(void)
{
   // SPI-Takt festlegen: SPI2X SPR1 SPR0
                     //     0    0    0    fosc/4 = 4 MHz bei 16MHz-Takt
                     //     1    0    0    fosc/2 = 8 MHz
                     //     1    0    1    fosc/8 = 2 MHz
                     //     0    0    1    fosc/16= 1 MHz
                     //     1    1    0    fosc/32= 500kHz
                     //     1    1    1    fosc/64= 250kHz
                     //     0    1    1    fosc/128=125kHz

   // SPI Control-Register: untere 2 bits SPR1, SPR0 fuer Takt-Teiler
   // SPCR = (0<<SPR1)|(0<<SPR0);

   // SPI Status-Register: unterstes bit SPI2X fuer Takt-Verdoppler
   // SPSR = (1<<SPI2X);

   // SPI 4MHz
   cbi(SPSR,SPI2X);
   cbi(SPCR,SPR1);
   cbi(SPCR,SPR0);

}



/*****************************************************************************
*   Function name : SPI_mode0
*   Returns :       None
*   Parameters :    None
*   Purpose :       set SPI mode 0
*****************************************************************************/
void spi_mode0(void)
{
   // CPOL      CPHA
   //   0         0   SPI mode 0
   //   0         1   SPI mode 1
   //   1         0   SPI mode 2
   //   1         1   SPI mode 3

   // SPI Control-Register: untere 2 bits CPOL, CPHA fuer Mode
   // SPCR = (0<<CPOL)|(0<<CPHA);

   // SPI Mode 0
   cbi(SPCR,CPOL);
   cbi(SPCR,CPHA);
/*   
   // SPI Mode 1
   cbi(SPCR,CPOL);
   sbi(SPCR,CPHA);
   
   // SPI Mode 2
   sbi(SPCR,CPOL);
   cbi(SPCR,CPHA);
   
   // SPI Mode 3
   sbi(SPCR,CPOL);
   sbi(SPCR,CPHA);
*/
}



/*****************************************************************************
*   Function name : SPI_mode1
*   Returns :       None
*   Parameters :    None
*   Purpose :       set SPI mode 1
*****************************************************************************/
void spi_mode1(void)
{
   // CPOL      CPHA
   //   0         0   SPI mode 0
   //   0         1   SPI mode 1
   //   1         0   SPI mode 2
   //   1         1   SPI mode 3

   // SPI Control-Register: untere 2 bits CPOL, CPHA fuer Mode
   // SPCR = (0<<CPOL)|(0<<CPHA);

   // SPI Mode 1
   cbi(SPCR,CPOL);
   sbi(SPCR,CPHA);

}



/*****************************************************************************
*   Function name : SPI_mode2
*   Returns :       None
*   Parameters :    None
*   Purpose :       set SPI mode 2
*****************************************************************************/
void spi_mode2(void)
{
   // CPOL      CPHA
   //   0         0   SPI mode 0
   //   0         1   SPI mode 1
   //   1         0   SPI mode 2
   //   1         1   SPI mode 3

   // SPI Control-Register: untere 2 bits CPOL, CPHA fuer Mode
   // SPCR = (0<<CPOL)|(0<<CPHA);

   // SPI Mode 2
   sbi(SPCR,CPOL);
   cbi(SPCR,CPHA);

}



/*****************************************************************************
*   Function name : SPI_mode3
*   Returns :       None
*   Parameters :    None
*   Purpose :       set SPI mode 3
*****************************************************************************/
void spi_mode3(void)
{
   // CPOL      CPHA
   //   0         0   SPI mode 0
   //   0         1   SPI mode 1
   //   1         0   SPI mode 2
   //   1         1   SPI mode 3

   // SPI Control-Register: untere 2 bits CPOL, CPHA fuer Mode
   // SPCR = (0<<CPOL)|(0<<CPHA);

   // SPI Mode 3
   sbi(SPCR,CPOL);
   sbi(SPCR,CPHA);

}



/*****************************************************************************
*   Function name : SPI_on
*   Returns :       None
*   Parameters :    None
*   Purpose :       Set SPI active for send/receive 
*****************************************************************************/
void spi_on(void)
{
   sbi(SPCR,SPE); // SPI aktiv schalten: CLK soll vorher korrekt sein
}



/*****************************************************************************
*   Function name : SPI_off
*   Returns :       None
*   Parameters :    None
*   Purpose :       Set SPI not active for send/receive 
*****************************************************************************/
void spi_off(void)
{
   cbi(SPCR,SPE); // SPI deaktiv schalten
}



/*****************************************************************************
*   Function name : SPI_send_rcv
*   Returns :       None
*   Parameters :    unsigned char data: byte to send and to receive
*   Purpose :       Send one byte through SPI and receive one
*                   1 Byte per SPI schicken, gleichzeitig 1 Byte empfangen
*****************************************************************************/
unsigned char spi_send_rcv(unsigned char out) // byte zum ADC/SRAM senden
                    // CS muss vorher low sein -> dies sicherstellen 
                    // DRDY muss low sein -> dies vorher sicherstellen 
                    // SCK muss zum SPI-Mode passen - z.B. low sein 
                    // SPI mode muss stimmen 
                    // SPI speed muss passen 
                    // SPI muss aktiv sein 
{
   unsigned char in;
   
   // SPI aktiv schalten
   //sbi(SPCR,SPE);   // SPI aktiv schalten: CLK,DRDY,Mode muss vorher passen

   SPDR = out;    // put byte 'out' in SPI data register for output
   while(!(SPSR & (1<<SPIF))); // wait for transfer complete, poll SPIF-flag
   //loop_until_bit_is_set(SPSR, SPIF);
   in = SPDR;     // read value in SPI data reg.
   
   // SPI deaktiv schalten
   //cbi(SPCR,SPE);   // SPI deaktiv schalten

   return in;     // return the byte clocked in from SPI slave
}



/*****************************************************************************
*   Function name : SPI_send_byte
*   Returns :       None
*   Parameters :    unsigned char data: byte to send
*   Purpose :       Send one byte through SPI
*                   1 Byte per SPI schicken
*   Funktion nicht noetig da SPI_send_rcv dies auch macht
*****************************************************************************/
/*void spi_send_byte(unsigned char byte)  // byte zum ADC/SRAM senden
                    // CS muss vorher low sein -> dies sicherstellen
                    // DRDY muss low sein -> dies vorher sicherstellen
                    // SCK muss zum SPI-Mode passen - z.B. low sein 
                    // SPI mode muss stimmen 
                    // SPI speed muss passen 
                    // SPI muss aktiv sein 
{
   // SPI aktiv schalten
   //sbi(SPCR,SPE);   // SPI aktiv schalten: CLK,DRDY,Mode muss vorher passen

   // bype per SPI senden
   SPDR = byte;     // put byte 'out' in SPI data register for output
   while(!(SPSR & (1<<SPIF)));   // wait for transfer complete, poll SPIF-flag
   //loop_until_bit_is_set(SPSR, SPIF);
   //dummy = SPDR;  // read value in SPI data reg.
                    // Byte nicht weiter genutzt
   
   // SPI deaktiv schalten
   //cbi(SPCR,SPE);   // SPI deaktiv schalten

} */



/*****************************************************************************
*   Function name : SPI_receive_byte
*   Returns :       unsigned char data: byte to send and to receive
*   Parameters :    None
*   Purpose :       Receive one byte through SPI (and send 0x00)
*                   1 Byte per SPI emofangen, gleichzeitig 0x00 senden
*   Funktion nicht noetig da SPI_send_rcv dies auch macht
*****************************************************************************/
/* unsigned char spi_receive_byte(void)  // byte vom ADC/SRAM empfangen
                    // DRDY muss dazu low sein -> dies vorher sicherstellen
                    // CS muss ebenfalls low sein -> dies vorher sicherstellen
                    // SCK muss vorher low sein -> dies vorher sicherstellen
                    // SPI mode muss stimmen 
                    // SPI speed muss passen 
                    // SPI muss aktiv sein 
{
   unsigned char r; // zu empfangendes Byte

   // SPI aktiv schalten
   //sbi(SPCR,SPE);   // SPI aktiv schalten: CLK,DRDY,Mode muss vorher passen
   
   // byte per SPI empfangen, dazu 0x00 senden
   SPDR = 0x00;     // put byte 'out' in SPI data register for output
   while(!(SPSR & (1<<SPIF)));   // wait for transfer complete, poll SPIF-flag
   //loop_until_bit_is_set(SPSR, SPIF);
   r = SPDR;        // read value in SPI data reg.

   // SPI deaktiv schalten
   //cbi(SPCR,SPE);   // SPI deaktiv schalten

   // empfangenes byte liefern 
   return r;

}*/

